/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

namespace fe
{

void SurfaceCopyOp::initialize(void)
{
	catalog< sp<Component> >("Input Surface");
	catalog< sp<Component> >("Template Surface");
	catalog< sp<Component> >("Output Surface");
	catalog<String>("Output Surface","IO")="output";
}

void SurfaceCopyOp::handle(Record& a_rSignal)
{
//	feLog("SurfaceCopyOp::handle\n");

	sp<SurfaceAccessibleI> spSurfaceInput=
			catalog< sp<Component> >("Input Surface");
	sp<SurfaceAccessibleI> spSurfaceTemplate=
			catalog< sp<Component> >("Template Surface");

	sp<SurfaceAccessibleI> spSurfaceOutput=surfaceOutput(a_rSignal);
	feLog("SurfaceCopyOp::handle spSurfaceOutput valid=%d\n",
			spSurfaceOutput.isValid());
	FEASSERT(spSurfaceOutput.isValid());

	sp<SurfaceAccessorI> spAccessor=spSurfaceTemplate->accessor(
			SurfaceAccessibleI::e_point,SurfaceAccessibleI::e_position);
	FEASSERT(spAccessor.isValid());

	U32 pointCount=spAccessor->count();
	feLog("SurfaceCopyOp::handle pointCount=%d\n",pointCount);

#if TRUE
	std::vector<SpatialTransform> transformArray;
	transformArray.resize(pointCount);

	for(U32 pointIndex=0;pointIndex<pointCount;pointIndex++)
	{
		SpatialTransform& rTransform=transformArray[pointIndex];
		setIdentity(rTransform);

		SpatialVector point=spAccessor->spatialVector(pointIndex);
		setTranslation(rTransform,point);
	}

	spSurfaceOutput->instance(spSurfaceInput,transformArray);
#else
	SpatialTransform transform;
	setIdentity(transform);

	for(U32 pointIndex=0;pointIndex<pointCount;pointIndex++)
	{
		SpatialVector point=spAccessor->spatialVector(pointIndex);
		setTranslation(transform,point);

		if(!(pointIndex%100))
		{
			feLog("SurfaceCopyOp::handle append %d/%d\n",
					pointIndex,pointCount);
		}
		spSurfaceOutput->append(spSurfaceInput,transform);
	}
#endif

//	feLog("SurfaceCopyOp::handle done\n");
}

} // namespace
